<?php
/**
 * Copyright 2020 JMango360
 */

/**
 * Class Jmango360_Japi_Model_Orchard_Sync
 */
class Jmango360_Japi_Model_Orchard_Sync
{
    const EVENT_ORDER_STATUS = 'order/status';

    protected $baseUrls;

    /**
     * Add job to queue
     *
     * @param array $data
     */
    public function addJob($data)
    {
        if (empty($data)) {
            return;
        }

        try {
            /** @var Mage_Core_Model_Resource $coreResource */
            $coreResource = Mage::getSingleton('core/resource');
            $writeAdapter = $coreResource->getConnection('core_write');
            $writeAdapter->insert(
                $coreResource->getTableName('japi_orchard_sync'),
                [
                    'data' => json_encode($data)
                ]
            );
            $this->log('Job create succeeded: ' . json_encode($data));
        } catch (\Exception $e) {
            $this->log('Job create failed: ' . $e->getMessage());
        }
    }

    /**
     * Run all jobs
     */
    public function runJob()
    {
        try {
            /** @var Mage_Core_Model_Resource $coreResource */
            $coreResource = Mage::getSingleton('core/resource');
            $connection = $coreResource->getConnection('core_read');
            $query = $connection->select()
                ->from($connection->getTableName('japi_orchard_sync'))
                ->limit(100);

            $result = $connection->fetchAll($query);
            if (count($result)) {
                $data = [];
                $jobIds = [];

                foreach ($result as $row) {
                    $jobIds[] = $row['id'];

                    try {
                        $d = json_decode($row['data'], true);
                        $d['jobId'] = $row['id'];

                        foreach ($d as $k => $v) {
                            if (is_array($v)) {
                                $this->prepareData($v, $data);
                            } else {
                                $this->prepareData($d, $data);
                                break;
                            }
                        }
                    } catch (\Exception $e) {
                        continue;
                    }
                }

                if (count($data)) {
                    foreach ($data as $type => $items) {
                        $this->init($type);

                        $response = $this->sync($type, array_values($items));

                        $this->log(sprintf('Job run result: %s', is_array($response) ? json_encode($response) : $response));

                        $error = false;
                        foreach ($response as $result) {
                            if ($result['error'] == 1) {
                                $error = true;
                            }
                        }

                        /**
                         * Clean jobs
                         */
                        if (!$error && count($jobIds)) {
                            $connection->delete(
                                $connection->getTableName('japi_orchard_sync'),
                                [$connection->quoteInto('id IN (?)', $jobIds)]
                            );
                        }
                    }
                }
            }
        } catch (\Exception $e) {
            $this->log('Job run failed: ' . $e->getMessage());
        }
    }

    /**
     * Orchard sync
     *
     * @param string $type
     * @param mixed $data
     * @return mixed
     * @throws
     */
    public function sync($type, $data)
    {
        if (!count($this->baseUrls)) {
            return array(
                'error' => 1,
                'message' => 'Orchard urls are missing'
            );
        }

        switch ($type) {
            case self::EVENT_ORDER_STATUS:
                $params['orderData'] = $data;
                break;
        }

        $headers = array(
            'Content-type: application/json'
        );

        $result = [];

        foreach ($this->baseUrls as $baseUrl) {
            try {
                $response = $this->_send('POST', $baseUrl, $params, $headers);

                $result[] = array(
                    'baseUrl' => $baseUrl,
                    'error' => 0,
                    'response' => $response
                );
            } catch (\Exception $e) {
                $result[] = array(
                    'baseUrl' => $baseUrl,
                    'error' => 1,
                    'message' => $e->getMessage()
                );
            }
        }

        return $result;
    }

    /**
     * Make HTTP request using cURL extension
     *
     * @param string $method
     * @param string $url
     * @param mixed $data
     * @param mixed $headers
     * @return string
     * @throws \Exception
     */
    protected function _send($method, $url, $data, $headers = array())
    {
        if (!function_exists('curl_init')) {
            throw new Exception('PHP cURL extension is missing.');
        }

        if (strtolower($method == 'get') && $data) {
            $url .= is_array($data) ? http_build_query($data) : $data;
        }

        //init and config request
        $ch = curl_init($url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_TIMEOUT, 5);

        if (strtolower($method) == 'post' && $data) {
            curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($data));
        }

        if ($headers) {
            curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
        }

        //log
        $this->log(sprintf('%s %s %s', strtoupper($method), $url, is_array($data) ? json_encode($data) : $data));

        //execute
        $response = curl_exec($ch);
        $responseCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        //get error if occurred
        $error = curl_error($ch);
        //free resources
        curl_close($ch);

        //log
        $this->log(sprintf('Response: [%s] %s', $responseCode, $response));

        //return
        if ($responseCode == 200) {
            return $response;
        } else {
            if ($error) {
                throw new Exception($error);
            } else {
                throw new Exception($responseCode);
            }
        }
    }

    /**
     * Init target URLs
     *
     * @param string $type
     */
    public function init($type = '')
    {
        $this->baseUrls = [];

        /** @var Jmango360_Japi_Helper_Hook $hookHelper */
        $hookHelper = Mage::helper('japi/hook');

        switch ($type) {
            case self::EVENT_ORDER_STATUS:
                $hooks = $hookHelper->getHooks(true);
                foreach ($hooks as $hook) {
                    if (!empty($hook['scope']) && $hook['scope'] == self::EVENT_ORDER_STATUS && !empty($hook['destination'])) {
                        $this->baseUrls[] = $hook['destination'];
                    }
                }
        }
    }

    /**
     * Prepare job data
     *
     * @param mixed $data
     * @param mixed $output
     * @return mixed
     */
    protected function prepareData($data, &$output)
    {
        if (!is_array($data)) {
            return $output;
        }

        if (empty($data['type'])) {
            return $output;
        }

        $type = $data['type'];

        if (!isset($output[$type])) {
            $output[$type] = [];
        }

        switch ($data['type']) {
            case self::EVENT_ORDER_STATUS:
                $key = '' . $data['orderId'];
                $output[$type][$key] = $data;
        }

        return $output;
    }

    /**
     * Log to file
     *
     * @param string $message
     */
    protected function log($message)
    {
        Mage::log($message, null, 'japi_orchard_sync.log');
    }
}