<?php
/**
 * Copyright 2019 JMango360
 */

/**
 * Class Jmango360_Japi_Model_Server
 */
class Jmango360_Japi_Model_Server extends Mage_Api2_Model_Server
{
    const SESSIONIDACTIONNAME = 'getSession';
    const TOKENACTIONNAME = 'getToken';
    const CATEGORYCONTROLLERNAME = 'rest_category';
    const USETOKENPATH = 'japi/jmango_rest_api/use_token';
    const DISABLE_FLAT_PRODUCT_PATH = 'japi/jmango_rest_developer_settings/disable_flat_product';
    const JMANGO360_CACHE_TYPE = 'jmango360api';
    const JMANGO360_CACHE_TAG = 'JMANGO360_API';

    protected $_excludeActionFromTokenValidation = array(
        self::SESSIONIDACTIONNAME,
        self::TOKENACTIONNAME,
        'getPluginVersion',
        'getMagentoInfo',
        'updateTheme',
        'success'
    );

    protected $_excludeControllerFromTokenValidation = array(
        self::CATEGORYCONTROLLERNAME,
    );

    protected $_excludeActionFromTokenReturn = array(
        'getPluginVersion',
        'getMagentoInfo',
        'updateTheme'
    );

    /**
     * Excluded params from request cache key
     *
     * @var array
     */
    protected $_excludedParams = array(
        'SID',
        'XDEBUG_SESSION_START',
        'token'
    );

    /**
     * Cacheable APIs
     *
     * @var array
     */
    protected $_cacheAPIs = array(
        'japi/rest_catalog/getCategoryTree',
        'japi/rest_product/list',
        'japi/rest_product/detail'
    );

    protected $_controllerInstance = null;

    protected $_model;

    /**
     * Main function
     *
     * @return void
     * @throws
     */
    public function run()
    {
        /** @var $request Jmango360_Japi_Model_Request */
        $request = Mage::getSingleton('japi/request');
        /** @var $renderer Jmango360_Japi_Model_Renderer_Json */
        $renderer = Mage::getModel('japi/renderer_json');
        /** @var $response Jmango360_Japi_Model_Response */
        $response = Mage::getSingleton('japi/response');
        Mage::app()->setResponse($response);

        /**
         * Log API request for debugging by demand
         */
        if (Mage::getStoreConfigFlag('japi/jmango_rest_developer_settings/enable')) {
            if ($request->has('XDEBUG_SESSION_START')) {
                Mage::setIsDeveloperMode(true);
                ini_set('display_errors', 1);
            }

            $debug['uri'] = $request->getMethod() . ' ' . $request->getPathInfo();

            /* @var $session Mage_Core_Model_Session */
            $session = Mage::getSingleton('core/session');
            $debug['session_id'] = $session->getSessionId();

            $requestParams = $request->getParams();

            if (isset($requestParams['api_key'])){
                $requestParams['api_key'] = '*';
            }
            if (strpos($request->getPathInfo(), 'japi/rest_customer/login') !== false) {
                if (isset($requestParams['login'])) {
                    $requestParams['login']['password'] = '*';
                }
            }
            if (isset($requestParams['password'])){
                $requestParams['password'] = '*';
            }

            $debug['params'] = $requestParams;

            $this->_log($debug, 'japi_request.log');
        }

        /**
         * Set current store if exist
         */
        $storeId = $request->getParam('store_id', null);
        if (is_numeric($storeId) && $storeId > 0) {
            $this->_setCurrentStore($storeId);
            Mage::getSingleton('core/session')->setData('store_id', $storeId);
        } else {
            if ($storeId = Mage::getSingleton('core/session')->getData('store_id')) {
                $this->_setCurrentStore($storeId);
            }
        }

        /**
         * Check if the right session ID is set
         */
        try {
            if (!$this->_validateSessionId()) {
                throw new Jmango360_Japi_Exception(
                    Mage::helper('japi')->__('Session expired.'),
                    Jmango360_Japi_Model_Request::REST_SESSION_EXPIRED
                );
            }
        } catch (Exception $e) {
            return $this->renderException($e, $renderer, $response);
        }

        /**
         * Handle cache
         */
        if ($canCache = $this->_canCache($request)) {
            $cacheKey = $this->_getAPICacheKey();
            $convertedCacheKey = $this->_convertCacheKey($cacheKey);
            /** @var Mage_Core_Model_Cache $cache */
            $cache = Mage::getSingleton('core/cache');
            if ($data = $cache->load($convertedCacheKey)) {
                $cacheMetadatas = $cache->getFrontend()->getMetadatas($this->_getInternalCacheId($convertedCacheKey));
                /** @var $response Jmango360_Japi_Model_Response */
                $response = Mage::getSingleton('japi/response');
                $response->render($this->unserialize($data));
                $response->setHttpResponseCode(Jmango360_Japi_Model_Server::HTTP_OK);
                $response->setHeader('X-Jmango360-Cache', 'HIT', true);
                if (is_array($cacheMetadatas) && is_int($cacheMetadatas['expire'])) {
                    $cacheLifetime = $this->_getAPICacheLifetime();
                    $response->setHeader('X-Jmango360-Cache-Age', sprintf('%s/%s', $cacheLifetime - ($cacheMetadatas['expire'] - time()), $cacheLifetime), true);
                }
                Mage::app()->setResponse($response);
                return;
            }
        }

        foreach (Mage::app()->getWebsite()->getStores() as $store) {
            /** @var Mage_Core_Model_Store $store */

            /**
             * Storing Flat product config value
             */
            $session = Mage::getSingleton('core/session');
            $_flatConfig = Mage::getStoreConfigFlag(Mage_Catalog_Helper_Product_Flat::XML_PATH_USE_PRODUCT_FLAT);
            $session->setData('use_flat_product_' . $store->getId(), $_flatConfig);

            /**
             * Bypass flat product check
             * MPLUGIN-1777: Ignore when get related products
             * MPLUGIN-2661: bypass flat product check for divoza.com
             * MPLUGIN-2892: bypass for byhilke.nl
             */
            if ($request->getActionName() != 'getRelated' &&
                $request->getActionName() != 'getCrossSell' &&
                $request->getActionName() != 'getUpSell'
            ) {
                /**
                 * MPLUGIN-2954: Make generic solution for Catalog Flat Product feature
                 */
                if (Mage::getStoreConfigFlag(self::DISABLE_FLAT_PRODUCT_PATH)
                    && strpos($_SERVER['HTTP_HOST'], 'motodiffusion') === false
                    && strpos($_SERVER['HTTP_HOST'], 'divoza') === false
                    && strpos($_SERVER['HTTP_HOST'], 'byemy.wine') === false
                    && strpos($_SERVER['HTTP_HOST'], 'byhilke.nl') === false) {
                    $store->setConfig(Mage_Catalog_Helper_Product_Flat::XML_PATH_USE_PRODUCT_FLAT, 0);
                }
            }

            /**
             * Bypass flat category check
             */
            if (strpos($_SERVER['HTTP_HOST'], 'motodiffusion') === false) {
                $store->setConfig(Mage_Catalog_Helper_Category_Flat::XML_PATH_IS_ENABLED_FLAT_CATALOG_CATEGORY, 0);
            }

            /**
             * MPLUGIN-2038: Bypass Netzarbeiter_NicerImageNames modify image url
             */
            $store->setConfig('catalog/nicerimagenames/disable_ext', 1);

            /**
             * MPLUGIN-2541: Bypass FactFinder catalog navigation
             */
            $store->setConfig('factfinder/activation/navigation', 0);

            /**
             * MPLUGIN-2783: Bypass Amasty_Shopby amshopby/seo/redirects_enabled value
             */
            $store->setConfig('amshopby/seo/redirects_enabled', 0);

            /**
             * MPLUGIN-3089: Bypass Amasty_Shopby price filter by slider and custom price ranges
             */
            $store->setConfig('amshopby/general/price_type', 0);
            $store->setConfig('amshopby/general/use_custom_ranges', 0);

            /**
             * MPLUGIN-3758: Bypass Phpro_Translate_Helper_Data::allowedToLogString()
             */
            $store->setConfig('translate/general/translation_logging', 0);
        }

        /**
         * Check mobile version for some deprecated functions
         */
        $mobileVersion = $request->getParam('version');
        $isOfflineCart = $mobileVersion ? version_compare($mobileVersion, '2.9', '<') : true;
        Mage::getSingleton('core/session')->setIsOffilneCart($isOfflineCart);

        /**
         * Check customer available
         */
        try {
            $customerId = Mage::getSingleton('customer/session')->getCustomerId();
            if ($customerId != null
                && Mage::getModel('customer/customer')->load($customerId)->getId() != $customerId) {
                Mage::getSingleton('customer/session')->setCustomerId(null);
                throw new Jmango360_Japi_Exception(
                    Mage::helper('japi')->__('Your account is not exist in the system. Please contact with System Administrator.'),
                    Jmango360_Japi_Model_Request::HTTP_BAD_REQUEST
                );
            }
        } catch (Exception $e) {
            return $this->renderException($e, $renderer, $response);
        }

        /**
         * Init empty admin user for Openwriter_Cartmart
         */
        if (Mage::helper('core')->isModuleEnabled('Openwriter_Cartmart')) {
            Mage::getSingleton('admin/session')->setUser(Mage::getModel('admin/user'));
        }

        /**
         * Custom rewrites
         */
        $rewrites = array();

        /**
         * ZOSUP-1279: Support Simtech_Searchanise
         */
        if (Mage::helper('core')->isModuleEnabled('Simtech_Searchanise')) {
            $rewrites = array_merge(array(
                'global/models/catalogsearch_resource/rewrite/fulltext_collection' => 'Jmango360_Japi_Model_Resource_Fulltext_Collection'
            ), $rewrites);
        }

        $config = Mage::app()->getConfig();
        foreach ($rewrites as $key => $value) {
            $config->setNode($key, $value);
        }

        try {
            /**
             * $response could have an exception in session or token check
             */
            if (!$response->isException()) {
                $this->_setRequest($request);
                $this->_setResponse($response);

                if (!$this->getRestDispatchModel() || !is_object($this->getRestDispatchModel())) {
                    throw new Jmango360_Japi_Exception(Mage::helper('japi')->__('No rest model found.'), self::HTTP_INTERNAL_ERROR);
                }

                $action = Mage::app()->getRequest()->getActionName();
                if (in_array($action, $this->_excludeActionFromTokenReturn)) {
                    $response->setSkipToken(true);
                }

                $request->setModel($this->getRestDispatchModel());
                /* @var $dispatcher Jmango360_Japi_Model_Dispatcher */
                $dispatcher = Mage::getModel('japi/dispatcher');
                $dispatcher->dispatch($request, $response);

                /**
                 * Cache response
                 */
                if ($canCache) {
                    try {
                        $this->_saveCache(
                            $this->serialize($response->getRawBody()),
                            $cacheKey,
                            $this->_getAPICacheTags($request, $response),
                            $this->_getAPICacheLifetime()
                        );
                    } catch (Exception $e) {
                        // ignore
                    }
                }
            }
        } catch (Exception $e) {
            $this->renderException($e, $renderer, $response);
        }

        /**
         * Remove objects from core/session
         *  -- objects cannot be serialized and the core/session is written/=serialized in the
         *     Mage_Core_Model_Resource_Session::__destruct
         */
        $this->unsRequest();
        $this->unsResponse();
    }

    /**
     * Get API cache lifetime
     *
     * @return int
     */
    protected function _getAPICacheLifetime()
    {
        $lifetime = (int)Mage::getStoreConfig('japi/jmango_rest_developer_settings/cache_lifetime');
        return $lifetime > 0 ? $lifetime : 3600;
    }

    /**
     * Prepare unified valid identifier with prefix
     * Source: Mage_Core_Model_Cache::_id()
     *
     * @param string $id
     * @return  string
     */
    protected function _getInternalCacheId($id)
    {
        if ($id) {
            $id = strtoupper($id);
        }

        return $id;
    }

    /**
     * Log proxy
     *
     * @param mixed $message
     * @param null $logFile
     */
    protected function _log($message, $logFile = null)
    {
        /** @var Jmango360_Japi_Helper_debug $logger */
        $logger = Mage::helper('japi/debug');
        $logger->log($message, $logFile);
    }

    /**
     * Save cache
     *
     * @param string $data
     * @param string $key
     * @param array $tags
     * @param null $lifetime
     */
    protected function _saveCache($data, $key, $tags = array(), $lifetime = null)
    {
        /** @var Mage_Core_Model_Cache $cache */
        $cache = Mage::getSingleton('core/cache');
        $newKey = $this->_convertCacheKey($key);
        if ($cache->save($data, $newKey, $tags, $lifetime)) {
            $debug = array(
                'Save' => $key,
                'To' => $newKey,
                'Tags' => join(',', $tags)
            );
            $this->_log($debug, 'japi_cache.log');
        }
    }

    /**
     * Get key to save to cache storage
     *
     * @param $key
     * @return string
     */
    protected function _convertCacheKey($key)
    {
        return 'JMANGO360_' . sha1($key);
    }

    /**
     * Check if API can cache
     *
     * @param Mage_Core_Controller_Request_Http $request
     * @return bool
     */
    protected function _canCache($request)
    {
        if (!Mage::getStoreConfigFlag('japi/jmango_rest_developer_settings/enable_api_cache')) {
            return false;
        }

        if (!Mage::app()->useCache(self::JMANGO360_CACHE_TYPE)) {
            return false;
        }

        return in_array(trim($request->getPathInfo(), ' /'), $this->_cacheAPIs);
    }

    /**
     * Serialize data
     *
     * @param $data
     * @return string
     * @throws
     */
    protected function serialize($data)
    {
        if (is_string($data)) {
            return $data;
        } elseif (is_array($data)) {
            return Mage::helper('japi/serialize')->serialize($data);
        } else {
            throw new Exception('Couldn\'t serialize value');
        }
    }

    /**
     * Unserialize data
     *
     * @param $data
     * @return array
     * @throws
     */
    protected function unserialize($data)
    {
        if (is_string($data)) {
            return Mage::helper('japi/serialize')->unserialize($data);
        } elseif (is_array($data)) {
            return $data;
        } else {
            throw new Exception('Couldn\'t unserialize value');
        }
    }

    /**
     * Get API cache tags
     *
     * @param Jmango360_Japi_Model_Request $request
     * @param Jmango360_Japi_Model_Response $response
     * @return array
     */
    protected function _getAPICacheTags($request, $response)
    {
        $tags = array(self::JMANGO360_CACHE_TAG);
        $data = $response->getRawBody();
        $pathInfo = $request->getPathInfo();

        if (strpos($pathInfo, 'japi/rest_product/detail') !== false) {
            if (isset($data['product']['product_id'])) {
                $tags[] = 'PRODUCT_' . $data['product']['product_id'];
            }
        } elseif (strpos($pathInfo, 'japi/rest_catalog/getCategoryTree') !== false) {
            $tags[] = 'CATEGORY';
        } elseif (strpos($pathInfo, 'japi/rest_product/list') !== false) {
            if (is_array($data['products'])) {
                foreach ($data['products'] as $product) {
                    if (!empty($product['product_id'])) {
                        $tags[] = 'PRODUCT_' . $product['product_id'];
                    }
                }
            }
        }

        return $tags;
    }

    /**
     * Generate cache key from request
     *
     * @return string
     * @throws
     */
    protected function _getAPICacheKey()
    {
        $parts = array();

        $store = Mage::app()->getStore();
        /** @var Mage_Customer_Model_Session $customerSession */
        $customerSession = Mage::getSingleton('customer/session');
        $request = Mage::app()->getRequest();

        // Add request uri
        $parts[] = trim($request->getPathInfo(), '/');
        // Add request params
        $params = $request->getParams();
        foreach ($params as $key => $value) {
            if (!in_array($key, $this->_excludedParams)) {
                $parts[] = sprintf('%s=%s', $key, $value);
            }
        }
        // Add customer group
        $parts[] = sprintf('cgid=%s', $customerSession->isLoggedIn() ? $customerSession->getCustomerGroupId() : 0);
        // Add store view
        $parts[] = sprintf('sid=%s', $store->getId());

        return join('|', $parts);
    }

    protected function _setCurrentStore($storeId)
    {
        if (!$storeId) return;
        if ($storeId == Mage::app()->getStore()->getId()) return;
        if ($storeId == -1) return;

        // Reset store ID
        Mage::app()->setCurrentStore($storeId);

        // Reset locale
        $locale = Mage::app()->getLocale();
        $locale->setDefaultLocale('');
        $locale->setLocaleCode($locale->getDefaultLocale());

        // Reset translator
        $transltor = Mage::app()->getTranslator();
        $transltor->setLocale($locale->getDefaultLocale());
        $area = $transltor->getConfig(Mage_Core_Model_Translate::CONFIG_KEY_AREA);
        $transltor->init($area, true);
    }

    public static function getApiTypes()
    {
        return self::$_apiTypes;
    }

    public function getRestDispatchModel()
    {
        return $this->_model;
    }

    public function setRestDispatchModel($model)
    {
        $this->_model = $model;
        return $this;
    }

    /**
     * Process thrown exception
     * Generate and set HTTP response code, error message to Response object
     *
     * @param Exception $exception
     * @param Mage_Api2_Model_Renderer_Interface $renderer
     * @param Jmango360_Japi_Model_Response $response
     * @return $this
     * @throws
     */
    protected function renderException(
        Exception $exception,
        Mage_Api2_Model_Renderer_Interface $renderer,
        Jmango360_Japi_Model_Response $response
    ) {
        if ($exception instanceof Jmango360_Japi_Exception && $exception->getCode()) {
            $httpCode = $exception->getCode();
        } else {
            $httpCode = self::HTTP_INTERNAL_ERROR;
        }

        try {
            //add last error to stack
            $response->setException($exception);

            $messages = array();

            /** @var Exception $exception */
            foreach ($response->getException() as $exception) {
                $message = array(
                    'code' => $exception->getCode(),
                    'message' => strip_tags(implode("\n", array_unique(explode("\n", $exception->getMessage()))))
                );

                //if (Mage::getIsDeveloperMode()) {
                $message['trace'] = $exception->getTraceAsString();
                //}

                $messages['messages']['error'][] = $message;
            }

            //set HTTP Code of last error, Content-Type and Body
            $response->setBody($renderer->render($messages));
            $response->setHeader('Content-Type', sprintf(
                '%s; charset=%s', $renderer->getMimeType(), Jmango360_Japi_Model_Response::RESPONSE_CHARSET
            ));
        } catch (Exception $e) {
            //tunnelling of 406(Not acceptable) error
            $httpCode = $e->getCode() == self::HTTP_NOT_ACCEPTABLE    //$e->getCode() can result in one more loop
                ? self::HTTP_NOT_ACCEPTABLE                      // of try..catch
                : self::HTTP_INTERNAL_ERROR;

            //if error appeared in "error rendering" process then show it in plain text
            $response->setBody($e->getMessage());
            $response->setHeader('Content-Type', 'text/plain; charset=' . Mage_Api2_Model_Response::RESPONSE_CHARSET);
        }

        /*
         * In the REST not registred HTTP codes are used. because this make reactions of netwerk devices and software unpredictable the error contains the REST status
         * -- but the real HTTP code is set to 500 (internal error)
         */
        if ($httpCode > 505) {
            $httpCode = 500;
        }

        $response->setHttpResponseCode($httpCode);

        return $this;
    }

    /**
     * Deprecated, but Orchard still used. So we avoid messing with Nexcessnet_Turpentine
     * by return random string instead of Magento form key
     *
     * @return string
     */
    public function getToken()
    {
        //return $this->_renewFormKey();
        return uniqid();
    }

    private function _validateToken()
    {
        if (!Mage::getStoreConfig(self::USETOKENPATH)) {
            return true;
        }

        $action = Mage::app()->getRequest()->getActionName();
        if (in_array($action, $this->_excludeActionFromTokenValidation)) {
            return true;
        }

        $controller = Mage::app()->getRequest()->getControllerName();
        if (in_array($controller, $this->_excludeControllerFromTokenValidation)) {
            return true;
        }

        if (Mage::getSingleton('core/session')->getIgnoreTokenCheck()) {
            Mage::getSingleton('core/session')->unsIgnoreTokenCheck();
            return true;
        }

        return $this->_validateFormKey();
    }

    private function _validateFormKey()
    {
        $test = Mage::getSingleton('core/session')->getFormKey();
        if (!($formKey = Mage::app()->getRequest()->getParam('token', null))
            || $formKey != Mage::getSingleton('core/session')->getFormKey()
        ) {
            return false;
        }
        return true;
    }

    private function _renewFormKey()
    {
        Mage::getSingleton('core/session')->setData('_form_key', '');
        return Mage::getSingleton('core/session')->getFormKey();
    }

    private function _validateSessionId()
    {
        /**
         * Action getSession renews the session ID and returns the new session ID
         * -- excluded from the check
         */
        $action = Mage::app()->getRequest()->getActionName();
        if (in_array($action, $this->_excludeActionFromTokenValidation)) {
            return true;
        }

        $requestSessionId = Mage::app()->getRequest()->getParam('SID');
        $sessionSessionId = Mage::getSingleton('core/session')->getSessionId();
        if ($requestSessionId != $sessionSessionId) {
            return false;
        }

        if (!Mage::getSingleton('core/session')->getIsRest()) {
            return false;
        }

        return true;
    }

    private function _setRequest($request)
    {
        Mage::getModel('core/session')->setServerRequest($request);
    }

    private function _setResponse($response)
    {
        Mage::getModel('core/session')->setServerResponse($response);
    }

    /**
     * @return Jmango360_Japi_Model_Request
     */
    public function getRequest()
    {
        return Mage::getModel('core/session')->getServerRequest();
    }

    /**
     * @return Jmango360_Japi_Model_Response
     */
    public function getResponse()
    {
        return Mage::getModel('core/session')->getServerResponse();
    }

    public function unsRequest()
    {
        Mage::getModel('core/session')->unsServerRequest();
    }

    public function unsResponse()
    {
        Mage::getModel('core/session')->unsServerResponse();
    }

    protected function _setIsRest()
    {
        Mage::getSingleton('core/session')->setIsRest(true);
        return $this;
    }

    public function setIsRest()
    {
        $this->_setIsRest();
        return $this;
    }

    public function getIsRest()
    {
        return Mage::getSingleton('core/session')->getIsRest() || Mage::app()->getRequest()->getModuleName() == 'japi';
    }

    public function unsetIsRest()
    {
        Mage::getSingleton('core/session')->setIsRest(false);
    }

    public function setIsSubmit()
    {
        Mage::getSingleton('core/session')->setIsSubmit(true);
    }

    public function getIsSubmit()
    {
        return (bool)Mage::getSingleton('core/session')->getIsSubmit();
    }

    public function unsetIsSubmit()
    {
        Mage::getSingleton('core/session')->setIsSubmit(false);
    }

    public function setControllerInstance($instance)
    {
        $this->_controllerInstance = $instance;
    }

    public function getControllerInstance()
    {
        return $this->_controllerInstance;
    }

    private function _sendExceptionResponse(
        Exception $e,
        Mage_Api2_Model_Renderer_Interface $renderer,
        Jmango360_Japi_Model_Response $response
    ) {
        $this->renderException($e, $renderer, $response);
    }
}